autoFit <-
function(dat, dims, start=NULL, criterion = "AIC", quietly = FALSE, r = TRUE, search = "basic", tol = 1e-2) {
  if (length(dim(dat)) > 0) n = dim(dat)[2]-1
  else if (!missing(dims)) n = length(dims)
  else stop("Must provde dimensions of model")

  if (missing(dims)) dims = apply(dat, 2, function(x) max(x)+1)[seq_len(n)]
  else if (is.vector(dat)) {
    dat = cbind(combinations(dims), dat)
    vnames = character(n)
    for (i in seq_len(n)) vnames[i] = paste("x", i, sep="")
    colnames(dat)[seq_len(n)] = vnames
  }

  vnames = colnames(dat)[-(n+1)]

  if (is.null(start)) {
    graph = makeGraph(n, vnames=vnames)
  }
  else graph = start

  best.fit = fitADMG(dat, graph, tol=tol, quietly=TRUE, sparse=FALSE, r = r)
  stat = switch(criterion, AIC=summary(best.fit, fisher=FALSE)$AIC, BIC=summary(best.fit, fisher=FALSE)$BIC)

#  tmp = searchADMG(emptygraph, gss, summary(f)$AIC)

  ok = TRUE

  while(ok) {

  tmp = switch(search,  basic = .searchADMG(best.fit$graph, dat, stat, criterion = criterion, quietly=quietly, tol=tol),
              full = .searchADMG2(best.fit$graph, dat, stat, criterion = criterion, quietly=quietly, tol=tol),
              exhaustive = stop("not implemented"))

    ok = tmp$moved
    if (ok) {
      best.fit = tmp
      stat = tmp$stat
      if (!quietly) print(best.fit)
    }
  }

  return(best.fit)
}
