fitADMG <-
function(dat, graph, r = TRUE, tol = sqrt(.Machine$double.eps), sparse = FALSE, quietly = TRUE) {
  n = graph$n
  if (tol < 0) stop("Tolerance must be positive")

  if(is.data.frame(dat)) {
    dat = plyr::daply(dat, seq_len(n), function(x) x[[n+1]])
  }

  dims = dim(dat)

  params = getMoebius(graph, makeptable(graph, dims=dims), r=r)
  if (!quietly) cat("Getting maps\n")
  maps = .getMaps(graph, sparse = sparse, dims = dims, r=r)

  ## # ORDER DATA AND ADD 0s IF NECESSARY
  ## dat = orderData(dat)
  ## new.dat = cbind(combinations(dims), 0)
  ## rows = prodlim::row.match(dat[,seq_len(n)], new.dat[,seq_len(n)])
  ## new.dat[rows, n+1] = dat[, n+1]
  ## dat = array(new.dat[,n+1], dim=dims)

  # SUFFICIENT STATISTICS ARE COUNTS OVER D \cup pa(D), FOR DISTRICT D.
  # WRITE AS A LIST OF THESE.
  dist.dat = list()
  for (d in seq_along(maps$dists)) {
    dist.dat[[d]] = margin.table(dat, maps$pa.dists[[d]])
    if (isTRUE(any(dist.dat[[d]] == 0))) warning("Zero counts in sufficient statistics")
  }

  # INITIAL TOLERANCE  FOR .doone
  tol2 = 1e-1
  move = 2*tol

  if (!quietly) cat("Fitting: ")
  count = 1

  while (move > tol || tol2 > sqrt(.Machine$double.eps)) {
    new.params = .doone(dist.dat, graph, params, maps, tol = tol2, quietly = quietly)
    move = .logLik(dist.dat, new.params$q, maps) - .logLik(dist.dat, params$q, maps)
    params = new.params
    tol2 = tol2/10
    if (!quietly) cat(count," ")
    count = count+1
  }
  if (!quietly) cat("\n")

  out = list(params = params, ll = .logLik(dist.dat, params$q, maps), dat=dat, graph=graph, dim=dims, maps=maps, r=r)
  class(out) = "mixed.fit"
  out
}
