\name{an}
\alias{an}
\alias{de}
\alias{dis}
\alias{nhd}
\alias{ant}
\alias{pa}
\alias{ch}
\alias{sp}
\alias{ne}
\alias{un}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Functions to explore graphical structure.
}
\description{
These functions may be used to explore the structure of graphs, and in particular to find the ancestors, descendants, parents, children, spouses and neighbours of a set of vertices \code{v}.  Further, they will find the district containing \code{v}, and the set of vertices which are anterior to \code{v}.
}
\usage{
an(graph, v = seq_len(n))
de(graph, v = seq_len(n))
dis(graph, v = seq_len(n))
nhd(graph, v = seq_len(n))
ant(graph, v = seq_len(n))

pa(graph, v = seq_len(n))
ch(graph, v = seq_len(n))
sp(graph, v = seq_len(n))
ne(graph, v = seq_len(n))

un(graph, strict = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{graph}{
A mixed graph, an object of type \code{graph}.
}
  \item{v}{
An integer vector containing the vertices in \code{graph} which the operation is to be performed on.  The default is the entire graph.
}
  \item{strict}{
  logical: if false (the default) vertices which have no arrowheads pointing to them but no undirected edges either are considered as being in the undirected component of the graph.
}
}
\details{
Detailed definitions can be found in Richardson (2009) and Evans and Richardson (2010).

\code{an()} finds the ancestors of \code{v}.  
\code{de()} finds the descendants of \code{v}.  
\code{dis()} finds the district(s) containing \code{v}.  
\code{nhd()} finds the neighbourhood(s) containing \code{v}.  
\code{ant()} finds the set of vertices anterior to \code{v}.  
\code{pa()} finds the parents of \code{v}.  
\code{ch()} finds the children of \code{v}.  
\code{sp()} finds the spouses of \code{v}.  
\code{ne()} finds the neighbours of \code{v}.  
\code{un()} finds the undirected part of the graph.  

All the operations taking the argument \code{v} are applied disjunctively: that is to say that \code{pa(graph, c(1,2))} finds the parents of the first and second vertices separately, and combines the results into one vector.
}
\value{
Returns a sorted integer list containing the vertices from the corresponding operation.}
\references{
Richardson, T.S. (2009) - A factorization criterion for acyclic directed mixed graphs. \emph{UAI-09}.

Evans, R.J. and Richardson, T.S. (2010) - Fitting acyclic directed mixed graphs to binary data. \emph{UAI-10}.
}
\author{
Robin Evans
}
\note{

}

\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
}
\examples{
data(grVerma)
g = makeGraph(5, d.edges=list(c(2,1),c(4,2),c(5,3)),
       bi.edges=list(c(1,3),c(2,3),c(2,5),c(3,4),c(4,5)))
g

an(g, 1)
pa(g, 1)
sp(g, c(3,5))

un(grVerma)
un(grVerma, strict=TRUE)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{graphs}

