\name{barren}
\alias{barren}
\alias{sterile}
\title{
Find the barren or sterile subset of vertices.
}
\description{
Given a set of vertices \code{v} in a graph, barren finds those elements of  \code{v} which do not have any (non-trivial) descendants in \code{v}.  Sterile finds elements of \code{v} which do not have any children in 
\code{v}, but may have other descendants.
}
\usage{
barren(graph, v = seq_len(n))
sterile(graph, v = seq_len(n))
}
\arguments{
  \item{graph}{
  An object of class \code{graph} - the graph being considered.
}
  \item{v}{
A subset of the vertices of \code{graph}.
}
}
\value{
An integer vector containing the vertices in the barren (or sterile) subset.
}
\references{
Richardson (2009) - A factorization criterion for acyclic directed mixed 
graphs. \emph{UAI-09}

Evans and Richardson (2010) - Maximum likelihood fitting of acyclic directed mixed graphs to binary data. \emph{UAI-10}
}
\author{
Robin Evans
}
\examples{
g = makeGraph(5, d.edges=list(c(2,1),c(4,2),c(5,3)),
      bi.edges=list(c(1,3),c(2,3),c(2,5),c(3,4),c(4,5)))
g

barren(g)
barren(g, c(1,4,5))

sterile(g, c(1,4,5))
}
\keyword{graphs}

