\name{factorize}
\alias{factorize}
\alias{print.mixed.factorization}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Factorizes ADMG according to (recursive) global Markov property.
}
\description{
Applies the partition function found in Richardson (2009) to obtain a factorization of the marginal distribution over some vertices in an ADMG.
}
\usage{
factorize(graph, v = seq_len(n), r = TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{graph}{
An ADMG, object of class \code{graph}.
}
  \item{v}{
Integer vector of vertices whose marginal distribution is to be factorized according  to the graph.
}
  \item{r}{
logical indicating whether recursive factorization should be used.
}
}
\details{
The partition function given in Richardson (2009) allows sets of vertices in an ADMG to be decomposed according to heads.  Richardson et al (2012) gives a recursive factorization which allows modelling of Verma constraints; some of the distributions which result in this case are reweighted versions of the original distribution.  More details can be found in the papers.
}
\value{
Returns an object of class \code{mixed.factorization}.  This consists of a 
list:
  \item{heads}{Heads as removed by partition function.}
  \item{tails}{Corresponding tails.}
  \item{vnames}{Names of vertices from \code{graph}.}
}
\references{
Richardson, T.S. (2009) - A factorization criterion for acyclic directed mixed graphs. \emph{UAI-09}.

Richardson et al. (2012).
}
\author{
Robin Evans, Ilya Shpitser
}


\seealso{
\code{\link{headsTails}}
\code{\link{partition}}
}
\examples{
data(grVerma)

factorize(grVerma)
factorize(grVerma, r=FALSE)

# order of vertices can change for different subsets
data(gr2)
factorize(gr2, r=FALSE) # d before e
factorize(gr2, c(3,4,5), r=FALSE) # e before d
}

\keyword{graphs}

