\name{getMoebius}
\alias{getMoebius}
\alias{print.mparam}
\title{
Evaluate generalized Moebius parameters
}
\description{
Given a full probability distribution over vertices of a graph, returns the value of the associated generalized Moebius parameters.
}
\usage{
getMoebius(graph, ptable, r = TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{graph}{
An object of class \code{graph}, an ADMG.
}
  \item{ptable}{
An array containing a joint probability distribution over the vertices of \code{graph}.
}
  \item{r}{
logical - should recursive factorizations be used?
}
}
\details{

}
\value{
An object of class \code{Mparams}.  That is, a list containing:
\item{q}{The Moebius parameters.  \code{q[[d]][[i]][[j]]} is a numeric array containing values of \eqn{P(X_H = x_H \,|\, X_T = j)} for each \eqn{x_H}{x_H}, where \eqn{H}{H} is the \code{i}th head in district \code{d}, and \code{j} indexes the tail states \eqn{x_T}{x_T}.}
\item{heads}{List of lists, sorted by district, and each sub-list containing integer vectors of the heads in that district.}
\item{tails}{List of lists, containing integer vectors of the tails corresponding to the heads above.}
\item{vnames}{Vector of names of vertices in graph.}
\item{dim}{Number of categories for each variable.}
\item{r}{as input}
}
\references{
Evans and Richardson (2010)
}
\author{
Robin Evans
}
\note{

}

\section{Warning }{This function does not verify that the given distribution satisfies the conditions of the model for the maps being calculated, and thus the distribution will not necessarily be recovered by mapping back using \code{getProbs}.

Note this function will not generally return correct values for parameters in the recursive parametrization unless all probabilities are equal.}

\seealso{
\code{\link{getProbs}}.
}
\examples{
data(gr2)

# Distribution of complete independence
ptable = array(1/32, rep(2,5))
getMoebius(gr2, ptable, r=TRUE)
}

