\name{maximalHeads}
\alias{maximalHeads}
\title{
Find maximal heads in an ADMG.
}
\description{
Given an ADMG, this finds which heads are maximal with respect to the criterion appropriate for the factorization (see details below)
}
\usage{
maximalHeads(graph, heads, r = TRUE)
}
\arguments{
  \item{graph}{
an object of class \code{graph}, containing an ADMG.
}
  \item{heads}{
A list of heads for \code{graph}.
}
  \item{r}{
Logical indicating whether or not recursive factorizations should be used.
}
}
\details{
The heads can be obtained using \code{\link{headsTails}}.

For the recursive factorization, the heads are partially ordered as \eqn{H_1 \prec H_2}{H_1 < H_2} if the intrinsic closure of \eqn{H_1}{H_1} is contained within the intrinsic closure of \eqn{H_2}{H_2}.

For the non-recursive case, \eqn{H_1 \prec H_2}{H_1 < H_2} if \eqn{H_1}{H_1} and its \code{\link{distail}} are contained within \eqn{H_2} and its distail.
}
\value{
A list of integer vectors, each containing a maximal head.
}
\references{
Evans (2011),
Richardson et al. (2012).
}
\author{
Ilya Shpitser, Robin Evans
}

\seealso{
\code{\link{headsTails}}.
}
\examples{
data(gr1)
h = headsTails(gr1)$heads

maximalHeads(gr1, h)
}

\keyword{graphs}

